@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_main.c,v $ $Revision: 3497 $ $Date: 2010-03-22 17:07:07 +0100 (Mon, 22 Mar 2010) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_main.c,v $ $Revision: 3791 $ $Date: 2010-09-13 08:01:29 +0200 (Mon, 13 Sep 2010) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <signal.h>
@@ -598,6 +598,7 @@ doit(arg)
 	char reqbuf[REQBUFSZ-3*LONGSIZE];
 	int set_authorization_id = 1;
 	struct dpm_srv_thread_info *thip = (struct dpm_srv_thread_info *) arg;
+	char *user_ca = NULL;
 	char *voname = NULL;
 
 	(void) Cgetnetaddress (thip->s, NULL, 0, &na_key, &clientip, &clientname, 0, 0);
@@ -666,6 +667,7 @@ doit(arg)
 	}
 #ifdef VIRTUAL_ID
 	if (thip->Csec_uid == -1) {
+		user_ca = Csec_server_get_client_ca (&thip->sec_ctx);
 #ifdef USE_VOMS
 		voname = Csec_server_get_client_vo (&thip->sec_ctx);
 		fqan = Csec_server_get_client_fqans (&thip->sec_ctx, &nbfqans);
@@ -673,8 +675,6 @@ doit(arg)
 		/* must reset VOMS pointers in Cns client context */
 		Cns_client_setAuthorizationId (thip->Csec_uid, thip->Csec_gid,
 		    thip->Csec_mech, thip->Csec_auth_id);
-		if (voname && fqan)
-			Cns_client_setVOMS_data (voname, fqan, nbfqans);
 
 		thip->Csec_nbgids = nbfqans ? nbfqans : 1;
 		if ((thip->Csec_gids =
@@ -686,14 +686,19 @@ doit(arg)
 			thip->s = -1;
 			return NULL;
 		}
-		if (Cns_getidmap (thip->Csec_auth_id, nbfqans,
+		if (Cns_getidmapc (thip->Csec_auth_id, user_ca, nbfqans,
 		    (const char **)fqan, &thip->Csec_uid, thip->Csec_gids)) {
-			dpmlogit (func, "[%s] (%s): Could not get virtual id for (%s,\"%s\"): %s !\n",
-			    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id, sstrerror (serrno));
+			if (serrno != EACCES)
+				dpmlogit (func,
+				    "[%s] (%s): Could not get virtual id for (%s,\"%s\"): %s !\n",
+				    clientip, clientname, thip->Csec_mech, thip->Csec_auth_id,
+				    sstrerror (serrno));
 			sendrep (thip->s, DPM_RC, serrno);
 			thip->s = -1;
 			return NULL;
 		}
+		if (voname && fqan)
+			Cns_client_setVOMS_data (voname, fqan, nbfqans);
 	}
 #endif
 	if (set_authorization_id) {
